Subject: Re: [PATCH] Add native windows on arm64 support

From: Dave Cramer <[email-masked]>
Date: Tue, 23 Jan 2024 08:46:37 -0500

On Tue, 19 Sept 2023 at 23:48, Michael Paquier <[email-masked]> wrote:

> On Tue, Sep 19, 2023 at 01:35:17PM +0100, Anthony Roberts wrote:
> > Was there an explicit request for something there? I was under the
> > impression that this was all just suggestion/theory at the moment.
>
> Yes.  The addition of a buildfarm member registered into the community
> facilities, so as it is possible to provide back to developers dynamic
> feedback to the new configuration setup you would like to see
> supported in PostgreSQL.  This has been mentioned a few times on this
> thread, around these places:
>
> https://www.postgresql.org/message-id/20220322103011.i6z2tuj4hle23wgx@jrouhaud
>
> https://www.postgresql.org/message-id/[email-masked]
>
> https://www.postgresql.org/message-id/[email-masked]
> --
> Michael
>

The attached patch works with v17. I will work on getting a buildfarm
animal up shortly

---

From: Dave Cramer <[email-masked]>
Date: Tue, 23 Jan 2024 16:13:05 -0500

> The attached patch works with v17. I will work on getting a buildfarm
I can build using meson manually, but for some reason building with the
buildfarm fails.

I'm not sure which files to attach to this to help. Andrew, what files do
you need ?

Dave

---

From: Michael Paquier <[email-masked]>
Date: Wed, 24 Jan 2024 08:32:08 +0900

>> The attached patch works with v17. I will work on getting a buildfarm

Thanks for doing that!

> I'm not sure which files to attach to this to help. Andrew, what files do

Probably build.ninja and the contents of meson-logs/ would offer
enough information?
--
Michael

---

From: Dave Cramer <[email-masked]>
Date: Wed, 24 Jan 2024 06:45:21 -0500

> Probably build.ninja and the contents of meson-logs/ would offer

I managed to get it to build the vcvarsall arch needs to be x64. I need to
add some options, but the patch above needs to be applied to build it.

Dave

---

From: Michael Paquier <[email-masked]>
Date: Thu, 25 Jan 2024 09:02:30 +0900

> add some options, but the patch above needs to be applied to build it.

Nice.  If I may ask, what kind of host and/or configuration have you
used to reach a state where the code can be compiled and run tests
with meson?  If you have found specific steps, it may be a good thing
to document that on the wiki, say around [1].

Perhaps you have not included TAP?  It may be fine in terms of runtime
checks and coverage.

[1]: https://wiki.postgresql.org/wiki/PostgreSQL_Buildfarm_Howto#Running_on_Windows
--
Michael

---

From: Dave Cramer <[email-masked]>
Date: Thu, 25 Jan 2024 08:31:07 -0500

> to document that on the wiki, say around [1].

I am running a mac-mini M2 with parallels running windows pro 11
The only thing required is this patch. Running in a native developer
prompt

meson setup build --prefix=c:\postgres
cd build
ninja
ninja install
ninja test

all run without errors
I also have buildfarm running and will run that today

Dave

> Perhaps you have not included TAP?  It may be fine in terms of runtime

Does TAP have to be explicitly added to the meson build ?

Dave

---

From: Dave Cramer <[email-masked]>
Date: Thu, 25 Jan 2024 08:45:52 -0500

> Does TAP have to be explicitly added to the meson build ?

I tried running my buildfarm using my git repo and my branch, but get the
following error
Status Line: 492 bad branch parameter
Content:
bad branch parameter fix_arm

Web txn failed with status: 1

---

From: Andrew Dunstan <[email-masked]>
Date: Thu, 25 Jan 2024 12:30:55 -0500

On 2024-01-24 We 19:02, Michael Paquier wrote:
> [1]:https://wiki.postgresql.org/wiki/PostgreSQL_Buildfarm_Howto#Running_on_Windows

I now have an ARM64 machine, so I set up a W11 ARM64 VM. I think we 
really want to build with x64_arm64, i.e. to generate native arm64 
binaries. Setting just x64 will not do that, AIUI.

I tried that with the buidfarm, setting that in the config file's call 
to PGBuild::VSenv::getenv().

That upset msvc_gendef.pl, so I added this there to keep it happy:

    $arch = 'x86_64' if $arch eq 'aarch64';

After that things went ok until I got this:

[1453/2088] "link" @src/backend/postgres.exe.rsp
FAILED: src/backend/postgres.exe src/backend/postgres.pdb
"link" @src/backend/postgres.exe.rsp
    Creating library src\backend\postgres.exe.lib
storage_lmgr_s_lock.c.obj : error LNK2019: unresolved external symbol 
_mm_pause referenced in function perform_spin_delay
src\backend\postgres.exe : fatal error LNK1120: 1 unresolved externals

I haven't made further progress, but I will return to it in the next day 
or so.

While this will be nice to have, I think it won't really matter until 
there is ARM64 support in released versions of Windows Server. AFAICT 
they still only sell versions for x86_64

cheers

andrew

--
Andrew Dunstan
EDB:https://www.enterprisedb.com

---

From: Dave Cramer <[email-masked]>
Date: Thu, 25 Jan 2024 14:07:11 -0500

On Thu, 25 Jan 2024 at 12:30, Andrew Dunstan <[email-masked]> wrote:

> I now have an ARM64 machine, so I set up a W11 ARM64 VM. I think we really
> want to build with x64_arm64, i.e. to generate native arm64 binaries.
> Setting just x64 will not do that, AIUI.
>
> I tried that with the buidfarm, setting that in the config file's call to
> PGBuild::VSenv::getenv().
>
> there is ARM64 support in released versions of Windows Server. AFAICT they
> still only sell versions for x86_64
>

I need it to build clients. The clients need arm64 libraries to link against

Dave

---

From: Andrew Dunstan <[email-masked]>
Date: Thu, 25 Jan 2024 14:31:43 -0500

On 2024-01-25 Th 08:45, Dave Cramer wrote:
>
>
>
>
> I tried running my buildfarm using my git repo and my branch, but get 
> the following error

You can't use your own branch with the buildfarm, you need to let it set 
up its own branches.

cheers

andrew

---

From: Dave Cramer <[email-masked]>
Date: Thu, 25 Jan 2024 15:56:20 -0500

> You can't use your own branch with the buildfarm, you need to let it set

So I guess I have to wait until this patch is merged in ?

Dave

---

From: Dave Cramer <[email-masked]>
Date: Thu, 25 Jan 2024 16:01:28 -0500

> still only sell versions for x86_64

I've tried it with my patch attached previously and x64_arm64 and it works
fine. builds using the buildfarm as well.

Is there a definitive way to figure out if the binaries are x64_arm64

Dave

---

From: Anthony Roberts <[email-masked]>
Date: Thu, 25 Jan 2024 21:03:51 +0000

Hi David,

Unix "file" or "dumpbin /headers" in vcvarsall are your best bets.

Thanks,
Anthony

---

From: Andrew Dunstan <[email-masked]>
Date: Thu, 25 Jan 2024 16:14:25 -0500

On 2024-01-25 Th 15:56, Dave Cramer wrote:
>>     I tried running my buildfarm using my git repo and my branch, but
>>     get the following error
>     You can't use your own branch with the buildfarm, you need to let
>     it set up its own branches.
>
>

Not quite.

There's a switch that lets you test your own code. If you say 
--from-source /path/to/repo  it will run in whatever state the repo is 
in. It won't care what branch you are on, and it won't try to update the 
repo. It won't report the results to the server, but it will build and 
test everything just like in a regular run. (On the "eat your own dog 
food" principle I use this mode a lot.) If you use that mode you 
probably also want to specify --verbose as well.

cheers

andrew

---

From: Dave Cramer <[email-masked]>
Date: Thu, 25 Jan 2024 16:17:15 -0500

> Unix "file" or "dumpbin /headers" in vcvarsall are your best bets.

So there is another way, select the file in Windows Explorer and right
click, in the compatibility tab if the "Windows on ARM" is greyed out it is
an arm binary.

So far mine are not :(

Thanks,

Dave

---

From: Andrew Dunstan <[email-masked]>
Date: Thu, 25 Jan 2024 16:32:39 -0500

On 2024-01-25 Th 16:17, Dave Cramer wrote:
>
>
> On Thu, 25 Jan 2024 at 16:04, Anthony Roberts 
> <[email-masked]> wrote:
>
>     Hi David,
>
> click, in the compatibility tab if the "Windows on ARM" is greyed out 
> it is an arm binary.
>
> So far mine are not :(
>

Yeah, I think the default Developer Command Prompt for VS2022 is set up 
for x86 builds. AIUI you should start by executing "vcvarsall x64_arm64".

cheers

andrew

---

From: Dave Cramer <[email-masked]>
Date: Thu, 25 Jan 2024 16:52:30 -0500

> for x86 builds. AIUI you should start by executing "vcvarsall x64_arm64".

Yup, now I'm in the same state you are

Dave

---

From: Michael Paquier <[email-masked]>
Date: Fri, 26 Jan 2024 10:32:48 +0900

> Yup, now I'm in the same state you are

Wait a minute here.  Based on [1], x64_arm64 means you can use a x64
host and you'll be able to produce ARM64 builds, still these will not
be able to run on the host where they were built.  How much of the
patch posted upthread is required to produce such builds?  Basically 
everything from it, I guess, so as build dependencies can be
satisfied?

[1]: https://learn.microsoft.com/en-us/cpp/build/building-on-the-command-line?view=msvc-170
--
Michael

---

From: Andrew Dunstan <[email-masked]>
Date: Fri, 26 Jan 2024 07:36:25 -0500

> [1]: https://learn.microsoft.com/en-us/cpp/build/building-on-the-command-line?view=msvc-170

If you look at the table here x86 and x64 are the only supported host 
architectures. But that's OK, the x64 binaries will run on arm64 (W11 
ARM64 has x64 emulation builtin). If that didn't work Dave and I would 
not have got as far as we have. But you want the x64_arm64 argument to 
vcvarsall so you will get ARM64 output.

cheers

andrew

--
Andrew Dunstan
EDB: https://www.enterprisedb.com

---

From: Dave Cramer <[email-masked]>
Date: Fri, 26 Jan 2024 09:18:50 -0500

On Fri, 26 Jan 2024 at 07:36, Andrew Dunstan <[email-masked]> wrote:

> > [1]:
> https://learn.microsoft.com/en-us/cpp/build/building-on-the-command-line?view=msvc-170
>
>

I've rebuilt it using  x64_arm64 and with the attached (very naive patch)
and I still get an x64 binary :(

---

From: Andrew Dunstan <[email-masked]>
Date: Mon, 29 Jan 2024 10:35:51 -0500

On 2024-01-26 Fr 09:18, Dave Cramer wrote:
> I've rebuilt it using  x64_arm64 and with the attached (very naive 
> patch) and I still get an x64 binary :(

I am definitely getting ARM64 binaries (e.g. for initdb.exe the Windows 
on ARM compatibility setting is greyed out)

I'll try your patch.

cheers

andrew

---

From: Andrew Dunstan <[email-masked]>
Date: Mon, 29 Jan 2024 11:16:41 -0500

> patch) and I still get an x64 binary :(

With this patch I still get a build error, but it's different :-)

[1406/2088] "link" @src/backend/postgres.exe.rsp
FAILED: src/backend/postgres.exe src/backend/postgres.pdb
"link" @src/backend/postgres.exe.rsp
    Creating library src\backend\postgres.exe.lib

storage_lmgr_s_lock.c.obj : error LNK2019: unresolved external symbol 
spin_delay referenced in function perform_spin_delay

cheers

andrew

---

From: Dave Cramer <[email-masked]>
Date: Mon, 29 Jan 2024 11:20:18 -0500

Dave Cramer
www.postgres.rocks

> src\backend\postgres.exe : fatal error LNK1120: 1 unresolved externals

Did you add the latest lock.patch ?

Dave

---

From: Andrew Dunstan <[email-masked]>
Date: Tue, 30 Jan 2024 08:38:40 -0500

On 2024-01-29 Mo 11:20, Dave Cramer wrote:
>>     I've rebuilt it using  x64_arm64 and with the attached (very
>>     naive patch) and I still get an x64 binary :(
>
>
>     storage_lmgr_s_lock.c.obj : error LNK2019: unresolved external
>     symbol spin_delay referenced in function perform_spin_delay
>

I'm a bit confused about exactly what needs to be applied. Can you 
supply a complete patch to be applied to a pristine checkout that will 
let me build?

cheers

andrew

---

From: Dave Cramer <[email-masked]>
Date: Tue, 30 Jan 2024 09:50:10 -0500

On Tue, 30 Jan 2024 at 08:38, Andrew Dunstan <[email-masked]> wrote:

> I'm a bit confused about exactly what needs to be applied. Can you supply
> a complete patch to be applied to a pristine checkout that will let me
> build?
>
>
> cheers
>

See attached.

Dave

---

From: Andrew Dunstan <[email-masked]>
Date: Tue, 30 Jan 2024 16:56:19 -0500

On 2024-01-30 Tu 09:50, Dave Cramer wrote:
>>     www.postgres.rocks <http://www.postgres.rocks>
>>
>>
>>         storage_lmgr_s_lock.c.obj : error LNK2019: unresolved
>>         external symbol spin_delay referenced in function
>>         perform_spin_delay
>>
>     supply a complete patch to be applied to a pristine checkout that
>     will let me build?
>
>
>     cheers
>
>
> See attached.
>
>

No, that is what is giving me the error shown above (just tried again to 
be certain). And it's not surprising, as patch 2 #ifdef's out the 
definition of spin_delay().

If you can get a complete build with these patches then I suspect you're 
not doing a proper ARM64 build.

cheers

andrew

---

From: Dave Cramer <[email-masked]>
Date: Tue, 30 Jan 2024 17:54:01 -0500

> not doing a proper ARM64 build.
Okay I will look when I get home in a week

Dave

---

From: Andrew Dunstan <[email-masked]>
Date: Wed, 31 Jan 2024 10:20:54 -0500

On 2024-01-30 Tu 17:54, Dave Cramer wrote:
>>>>             I've rebuilt it using x64_arm64 and with the attached
>>>>             (very naive patch) and I still get an x64 binary :(
>>>
>>>
>>         I'm a bit confused about exactly what needs to be applied.
>>         Can you supply a complete patch to be applied to a pristine
>>         checkout that will let me build?
>>
>>
>>         cheers
>>
>>
>>     See attached.
>>
>>
>
>     No, that is what is giving me the error shown above (just tried
>     again to be certain). And it's not surprising, as patch 2 #ifdef's
>     out the definition of spin_delay().
>
>     If you can get a complete build with these patches then I suspect
>     you're not doing a proper ARM64 build.
>
>

I made some progress. The attached is mostly taken from 
<https://postgr.es/m/[email-masked]>

With it applied I was able to get a successful build using the buildfarm 
client. However, there are access violations when running some tests, so 
there is still some work to do, apparently.

cheers

andrew

--
Andrew Dunstan
EDB:https://www.enterprisedb.com
diff --git a/doc/src/sgml/installation.sgml b/doc/src/sgml/installation.sgml
index ed5b285a5e..d9b8649dab 100644
--- a/doc/src/sgml/installation.sgml
+++ b/doc/src/sgml/installation.sgml
@@ -4150,7 +4150,7 @@ make: *** [postgres] Error 1
    <sect3 id="install-windows-full-64-bit">
     <title>Special Considerations for 64-Bit Windows</title>
     <para>
-     PostgreSQL will only build for the x64 architecture on 64-bit Windows.
+     PostgreSQL will only build for the x64 and ARM64 architecture on 64-bit Windows.
     </para>
     <para>
      Mixing 32- and 64-bit versions in the same build tree is not supported.
diff --git a/meson.build b/meson.build
index 8ed51b6aae..14aea924ec 100644
--- a/meson.build
+++ b/meson.build
@@ -2046,8 +2046,11 @@ int main(void)
 elif host_cpu == 'arm' or host_cpu == 'aarch64'
 
   prog = '''
+#ifdef _MSC_VER
+#include <intrin.h>
+#else
 #include <arm_acle.h>
-
+#endif
 int main(void)
 {
     unsigned int crc = 0;
@@ -2061,7 +2064,11 @@ int main(void)
 }
 '''
 
-  if cc.links(prog, name: '__crc32cb, __crc32ch, __crc32cw, and __crc32cd without -march=armv8-a+crc',
+  if cc.get_id() == 'msvc'
+    cdata.set('USE_ARMV8_CRC32C', false)
+    cdata.set('USE_ARMV8_CRC32C_WITH_RUNTIME_CHECK', 1)
+    have_optimized_crc = true
+  elif cc.links(prog, name: '__crc32cb, __crc32ch, __crc32cw, and __crc32cd without -march=armv8-a+crc',
       args: test_c_args)
     # Use ARM CRC Extension unconditionally
     cdata.set('USE_ARMV8_CRC32C', 1)
diff --git a/src/include/storage/s_lock.h b/src/include/storage/s_lock.h
index 69582f4ae7..44abd42691 100644
--- a/src/include/storage/s_lock.h
+++ b/src/include/storage/s_lock.h
@@ -707,15 +707,31 @@ typedef LONG slock_t;
 
 #define SPIN_DELAY() spin_delay()
 
-/* If using Visual C++ on Win64, inline assembly is unavailable.
- * Use a _mm_pause intrinsic instead of rep nop.
+/*
+ * If using Visual C++ on Win64, inline assembly is unavailable.
+ * Use architecture specific intrinsics.
  */
 #if defined(_WIN64)
+/*
+ * For Arm64, use __isb intrinsic. See aarch64 inline assembly definition for details.
+ */
+#ifdef _M_ARM64
+static __forceinline void
+spin_delay(void)
+{
+	 /* Reference: https://learn.microsoft.com/en-us/cpp/intrinsics/arm64-intrinsics#BarrierRestrictions */
+	__isb(_ARM64_BARRIER_SY);
+}
+#else
+/*
+ * For x64, use _mm_pause intrinsic instead of rep nop.
+ */
 static __forceinline void
 spin_delay(void)
 {
 	_mm_pause();
 }
+#endif
 #else
 static __forceinline void
 spin_delay(void)
diff --git a/src/port/pg_crc32c_armv8.c b/src/port/pg_crc32c_armv8.c
index d47d838c50..f18903a68a 100644
--- a/src/port/pg_crc32c_armv8.c
+++ b/src/port/pg_crc32c_armv8.c
@@ -14,7 +14,9 @@
  */
 #include "c.h"
 
+#ifndef _MSC_VER
 #include <arm_acle.h>
+#endif
 
 #include "port/pg_crc32c.h"
 
diff --git a/src/tools/msvc_gendef.pl b/src/tools/msvc_gendef.pl
index 12c49ed265..dfb26916bd 100644
--- a/src/tools/msvc_gendef.pl
+++ b/src/tools/msvc_gendef.pl
@@ -120,9 +120,9 @@ sub writedef
 	{
 		my $isdata = $def->{$f} eq 'data';
 
 		# Strip the leading underscore for win32, but not x64
 		$f =~ s/^_//
-		  unless ($arch eq "x86_64");
+		  unless ($arch eq "x86_64" || $arch eq "aarch64");
 
 		# Emit just the name if it's a function symbol, or emit the name
 		# decorated with the DATA option for variables.
@@ -143,7 +143,7 @@ sub writedef
 sub usage
 {
 	die("Usage: msvc_gendef.pl --arch <arch> --deffile <deffile> --tempdir <tempdir> files-or-directories\n"
-		  . "    arch: x86 | x86_64\n"
+		  . "    arch: x86 | x86_64 | aarch64\n"
 		  . "    deffile: path of the generated file\n"
 		  . "    tempdir: directory for temporary files\n"
 		  . "    files or directories: object files or directory containing object files\n"
@@ -160,7 +160,7 @@ GetOptions(
 	'tempdir:s' => \$tempdir,) or usage();
 
 usage("arch: $arch")
-  unless ($arch eq 'x86' || $arch eq 'x86_64');
+  unless ($arch eq 'x86' || $arch eq 'x86_64' || $arch eq 'aarch64');
 
 my @files;

---

From: Peter Eisentraut <[email-masked]>
Date: Wed, 31 Jan 2024 16:34:13 +0100

> + PostgreSQL will only build for the x64 and ARM64 architecture on 

Are there any other 64-bit architectures for Windows?

Possibly, the original sentence was meant to communicate that ARM was 
not supported, in which case it could now be removed?

---
@@ -120,9 +120,9 @@ sub writedef
 		my $isdata = $def->{$f} eq 'data';
 
-		# Strip the leading underscore for win32, but not x64
+		# Strip the leading underscore for win32, but not x64 and aarch64
 		$f =~ s/^_//

From: Andrew Dunstan <[email-masked]>
Date: Wed, 31 Jan 2024 10:41:57 -0500

On 2024-01-31 We 10:34, Peter Eisentraut wrote:
>> - PostgreSQL will only build for the x64 architecture on 64-bit 
>> Windows. + PostgreSQL will only build for the x64 and ARM64 
>> architecture on 64-bit Windows.
>

x86? That is in fact the default setting for VS even on ARM64.

cheers

andrew